#include "TICPhysics.h"
#include "TFPMWPhysics.h"
#include <TChain.h>
#include <regex>

void HistoFiller(const char* Path);

void CorrectionZ(const char* Path){

    //===========================================================================================================
    if( Path &&  Path[0] != '\0'){
        HistoFiller(Path);
        return;
    }
    //===========================================================================================================


    TFile *f = new TFile("Output/histo_merged.root","open");
    //===========================================================================================================
    TH2F *hZvsTS = nullptr , *hZvsTheta = nullptr;
    f->GetObject("hZvsTS",hZvsTS);
    f->GetObject("hZvsTheta",hZvsTheta);
    //===========================================================================================================

    TCanvas *c = new TCanvas("c","c");
    c->Divide(2);
    c->cd(1);
    hZvsTheta->Draw("colz");
    c->cd(2);
    hZvsTS->Draw("colz");

} // end macro:

void HistoFiller(const char* Path){

    //===========================================================================================================
    // Import
    //===========================================================================================================
    TChain *chain = new TChain("PhysicsTree");
    chain->Add (Path);
    //===========================================================================================================
    TICPhysics* IC = new TICPhysics() ;
    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC); 

    TFPMWPhysics* FPMW = new TFPMWPhysics() ;
    chain->SetBranchStatus("FPMW", true);
    chain->SetBranchAddress("FPMW", &FPMW); 

    double FF_IC_X, VAMOS_TS_hour;
    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);

    chain->SetBranchStatus("VAMOS_TS_hour", true);
    chain->SetBranchAddress("VAMOS_TS_hour", &VAMOS_TS_hour);
    //===========================================================================================================

    //===========================================================================================================
    // Histo initialisation + filling just to get the range 
    TH2F *hZvsTheta = new TH2F("hZvsTheta","hZvsTheta",1000,-0.5,0.5,1000,20,80);
    TH2F *hZvsTS = new TH2F("hZvsTS","hZvsTS",2000,0,200,1000,20,80);
    //===========================================================================================================
    int Nentries = chain->GetEntries();   
    //int Nentries = 1e6;
    auto start = std::chrono::high_resolution_clock::now();

    for (int e = 0; e < Nentries; e++) {    
        if (e % 100000  == 0 && e > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }

        chain->GetEntry(e);

        if(FF_IC_X>-530 && VAMOS_TS_hour>0){
            hZvsTheta->Fill(FPMW->Thetaf,IC->Chio_Z);
            hZvsTS->Fill(VAMOS_TS_hour,IC->Chio_Z);
        }
    }

    //===========================================================================================================
    // Save file
    //===========================================================================================================

    string strPath(Path);    
    regex pattern("Run(\\d{3})");  // Regex to match Run followed by exactly 3 digits
    smatch matches;

    if (regex_search(strPath, matches, pattern)) {
        string number = matches[1];  // Extract the number
        string fileName = "Output/histo_" + number + ".root";  // Generate the filename

        // Create the TFile
        TFile* file = new TFile(fileName.c_str(), "RECREATE");
        if (file->IsOpen()) {
            cout << "File created: " << fileName << endl;
        } else {
            cout << "Failed to create the file." << endl;
        }

        hZvsTS->Write();
        hZvsTheta->Write();
        // Don't forget to close the file
        file->Close();
    } 
    else {
        cout << "Pattern not found!" << endl;
    }

}
